/*
importPackage(java.lang);
importPackage(java.util);
importPackage(java.io);
importPackage(java.net);
importPackage(java.sql);
importPackage(java.text);
*/
importClass(Packages.com.alarmpoint.integrationagent.apxml.APXMLMessage);
importClass(Packages.com.alarmpoint.integrationagent.apxml.APXMLToken);
importClass(Packages.com.thoughtworks.xstream.XStream);
importClass(Packages.com.thoughtworks.xstream.converters.reflection.PureJavaReflectionProvider);

load( "integrationservices/wunderground/helpers.js" );

var API_KEY = '123456ABAC';

var xStream = new XStream(new PureJavaReflectionProvider());

/**
 * This is the main method for handling APXML messages sent to the
 * Integration Service by APClient.
 */
function apia_input(apxml)
{
  // Return null because this sample integration is not expected to send or receive messages in this manner. 
  return null;
}

/**
 * This is the main method for handling APXML messages sent to the
 * Integration Service by xMatters.  The messages may be responses to
 * previous submissions, or they may be requests to the Integration Service
 * that originate from xMatters
 * (e.g., via an ExternalServiceMessage/ExternalServiceRequest2).
 * <p>
 * Any APXMLMessage object that this method returns will be sent to xMatters
 * either via the Integration Service's outbound queues or directly as a
 * Web Service response, depending on the mechanism used to submit the APXML.
 */
function apia_response(apxml)
{
  var method = apxml.getMethod();
  if (APXMLMessage.SEND_METHOD.equalsIgnoreCase(method))
  {
    return handleSend(apxml);
  }
  else
  {
    throw new IllegalArgumentException("Unrecognized APXML method: " + method);
  }
}


/**
 * The main routine for handling Requests starts here.
 * <p>
 * Throw an exception that will be logged by the Integration Agent.
 */
function handleSend(apxml)
{
  /*************************************************************************************************************/
  // This is the function that will handle successful queries from the Integrated Properties section on the form.
  // Any returned APXMLMessage object will be sent as the response from the IA to the form.
  
  // This value will always be sent as part of the request. You can use this value to dispatch the request
  // to one of several functions in this file.
  /*************************************************************************************************************/
  var action = apxml.getValue("xmattersRebRequestAction");
  
  // For this sample, we are providing the user with getIncident to call and fetch integrated properties.
  if (action == "getWeather") {

    return handleGetWeather(apxml);

  } else {
    ServiceAPI.getLogger().error("Unknown or unspecified xmattersRebRequestAction: " + action);
    return;
  }
}

function handleGetWeather(apxml) {

  // All responses must include this value from the request so that the response 
  // can be matched to its originating request.
  var requestUuid = apxml.getValue("xmattersRebRequestUuid");

  // The getEventProperties helper method will read the request properties out 
  // of the request.
  var obj = getEventProperties(apxml);

  // Create a minimal response
  var response = ServiceAPI.createAPXML();
  response.setMethod(APXMLMessage.RESPONSE_METHOD);
  response.setSubclass(APXMLMessage.RESPONSE_SUBCLASS);

  /*****************************************************************************/
  // This field is REQUIRED in the response. If omitted, xMatters will not be 
  // able to match the response to the request that originated it.
  /*****************************************************************************/
  response.setToken('xmattersRebRequestUuid', requestUuid, APXMLToken.Type.STRING);

  // Here we query an external data source for the values of the integrated properties
  var weatherJson = getWeatherData( obj[ 'Zip Code' ] );

  // Here we add some sample properties to the response.  These will be used to 
  // populate the response properties on the form, assuming properties with these 
  // names have been defined.
  var addTokens = {};
  addTokens[ 'Icon URL'      ] = weatherJson[ 'Icon URL'     ];
  addTokens[ 'High F'        ] = weatherJson[ 'High F'       ];
  addTokens[ 'Low F'         ] = weatherJson[ 'Low F'        ];
  addTokens[ 'High C'        ] = weatherJson[ 'High C'       ];
  addTokens[ 'Low C'         ] = weatherJson[ 'Low C'        ];
  addTokens[ 'Conditions'    ] = weatherJson[ 'Conditions'   ];
  addTokens[ 'Max Wind MPH'  ] = weatherJson[ 'Max Wind MPH' ];
  addTokens[ 'Max Wind KPH'  ] = weatherJson[ 'Max Wind KPH' ];

  /*****************************************************************************/
  // Add the response properties to the response message. The field MUST be 
  // called "additionalTokens", and the expected value should be a JSON 
  // representation of the values. Use JSON.stringify() to encode the values 
  // as JSON.
  /*****************************************************************************/
  response.setToken('additionalTokens', JSON.stringify(addTokens), APXMLToken.Type.STRING);

  ServiceAPI.getLogger().warn('Sending response with tokens ' + JSON.stringify(addTokens));

  // Return the response so that it gets sent back to xMatters.
  return response;
}


function getWeatherData( zipCode ) {
    
  // Set up the url for the HTTP GET
  var url = 'http://api.wunderground.com/api/' + API_KEY + '/forecast/q/' + zipCode + '.json';

  // httpget function from our helpers.js file
  weatherRAW = httpget( url );
  weather = JSON.parse( weatherRAW );
  
  // extractTomorrowForecast is also in helpers.js
  forecast = extractTomorrowForecast( weather.forecast.simpleforecast.forecastday )

  // Encapsulate the data into something readable
  data = {
    'Icon URL': forecast.icon_url,
    'High F': forecast.high.fahrenheit,
    'Low F': forecast.low.fahrenheit,
    'High C': forecast.high.celsius,
    'Low C': forecast.low.celsius,
    'Conditions': forecast.conditions,
    'Max Wind MPH': forecast.maxwind.mph,
    'Max Wind KPH': forecast.maxwind.kph
  }
  
  return data;
}




/**
 * The request properties are passed as "additionalTokens" in the request.  They appear as XML, so they must
 * be converted to a JavaScript object.
 */
function getEventProperties(apxml) {

  var evProps = apxml.getValue("additionalTokens");

  // deserialize the additional tokens from the XML
  if (evProps != null && evProps != "") {
    var addTokensLinkedHashMap = xStream.fromXML(evProps);
  } else {
    var addTokensLinkedHashMap = new HashMap();
  }

  // Construct a JavaScript object with the key/values defined in the XML.
  var obj = {};
  var keyIterator = addTokensLinkedHashMap.keySet().iterator();
  while (keyIterator.hasNext()) {
    key = keyIterator.next();
    obj[key] = addTokensLinkedHashMap.get(key);
  }

  return obj;
}


/*
 * Method to handle all accept responses from xMatters.
 *
 * This method will only be called when the token provideServiceMessages is set to TRUE.
 */
/*
function handleAccept(messageText, incidentId, assignee)
{

}
*/
/*
 * Method to handle all reject responses from xMatters.
 *
 * This method will only be called when the token provideServiceMessages is set to TRUE.
 */
/*
function handleReject(messageText, incidentId, assignee)
{

}
*/
/*
 * Method to handle all annotations from xMatters.
 *
 * This method will only be called when the token provideServiceMessages is set to TRUE.
 */
/*
function handleAnnotate(messageText, incidentId, assignee)
{

}
*/
